
program normal_modes
  use mat_module
  implicit none
  integer, parameter :: Na = 2      ! Natoms
  integer, parameter :: Dim    = 3  ! dimensionality--can put as 1 for line.
  integer, parameter :: Ntot   = Na * Dim

  real*8 U(Ntot, Ntot)
  real*8 B(Ntot, Ntot)
  real*8 B_evl(Ntot)

  real*8  mass(Na)
  integer mapAtom(Ntot)   ! see below.
  integer mapDim( Ntot)

  real*8  r0(dim, Na)

  open(3,file='normal_mod.inp', status='old')

  call maps
  call read_mass
  call read_approx_equilibirium
  call refine_equilibrium
  call make_U
  call make_B
  call diag_B
  call print_B

contains

  subroutine read_mass
    implicit none
    integer iatom, idummy
    write(6,*)' reading the masses from file normal_mod.inp '
    write(6,*)' give the masses in a.m.u. (mass(O)=16, mass(H)=1.00727) '
    write(6,*)' in format iatom, mass(iatom) '
    do iatom=1, Na
       read( 3,*)idummy, mass(iatom)
       
       call check(idummy, iatom,' idummy, iatom  ')  ! a routine to check that
                                                     ! 2 integers are equal.
    enddo

   write(6,*)' masses in a.m.u. ',mass
   mass = mass*1822.89
   write(6,*)' nmasses in a.u. ',mass

  end subroutine read_mass

  subroutine maps
    implicit none

    integer iatom, itot, id

    itot =0
    do iatom=1, Na
       do id=1, dim
          itot=itot+1
          mapAtom(itot) = iatom
          mapDim( itot) = id
       enddo
    enddo
    do itot=1, ntot
       write(6,*)' itot, associated atom ',itot,mapAtom(itot), mapDim(itot)
    enddo
 end subroutine

 subroutine read_approx_equilibirium
   implicit none
   
   integer iatom, idummy

   do iatom=1, Na

      read(3,*)idummy, r0(:, iatom)

      if(idummy/=iatom) then
         write(6,*)' problem in read_app_equib iatom, idummy ',iatom,idummy
         stop
      endif
      write(6,*)' r0 for atom iatom ',iatom, r0(:,iatom) 
   enddo
 end subroutine read_approx_equilibirium

 subroutine refine_equilibrium
   implicit none

!
!  here you'll insert a routine to optimize the geometry till the force=0.
!

 end subroutine refine_equilibrium

 subroutine make_U
   implicit none
   integer itot, id, iatom, jtot, jd,jatom
   real*8, parameter :: step = 1.d-5

   real*8   rup(dim, na)
   real*8 rdown(dim, na)
   real*8 force_up(dim, na)
   real*8 force_down(dim, na)

   do itot=1, ntot
      rup   = r0
      rdown = r0 
      id = mapdim(itot)
      iatom = mapatom(itot)

      rup(id, iatom) = &
      rup(id, iatom) + step

      rdown(id, iatom) = &
      rdown(id, iatom) - step

      call force_sub(rup, force_up)
      call force_sub(rdown, force_down)

      do jtot=1, ntot
         jd = mapdim(jtot)
         jatom = mapatom(jtot)

         U(itot, jtot) = (force_up(  jd,jatom)-&
                          force_down(jd,jatom))/2.d0/step
         
      end do
   enddo

   write(6,*)' plotting U '
   call plot_mat(U, 6)
 end subroutine make_U

 subroutine make_B
   implicit none
   integer itot, jtot

   do itot=1, Ntot
      do jtot=1, Ntot
         B(itot, jtot) = &
        -U(itot, jtot)/  &
                sqrt(mass(mapAtom(itot))*mass(mapatom(jtot)))
      end do
   end do
   write(6,*)' plotting B '
   call plot_mat(B, 6)
 end subroutine make_B

 subroutine diag_B
   implicit none
   real*8 B_vc(Ntot, Ntot)

   call mat_diag(B, B_vc, B_evl)
 end subroutine diag_B

 subroutine print_B

   implicit none
   real*8  ww
   integer itot

   do itot=1, ntot
      !
      ! Take sqrt of B
      !
      if(B_evl(itot)>0) then
         ww = sqrt(B_evl(itot))
      else if(B_evl(itot)>-1.d-10) then
         ww = 0.d0
      else
         write(6,*)' problem with itot, B_evl ',itot, B_evl(itot)
         stop
      endif

      write(6,*)' itot , ww ',itot,ww
   enddo
 end subroutine print_B
   

 subroutine force_sub(rr, force)
   implicit none

   real*8 rr(dim, Na), force(dim, Na)
   real*8 rup(dim, Na)
   real*8 rdown(dim, Na)
   integer itot, id, iatom
   real*8, parameter :: step = 1.d-5

   do itot=1, ntot
      rup = rr
      rdown = rr

      iatom = mapatom(itot)
      id    = mapdim( itot)

      rup(id, iatom)   = rup(  id, iatom) + step
      rdown(id, iatom) = rdown(id, iatom) - step
     
      force(id, iatom) = - (pot(rup) - pot(rdown))/2.d0/step

   enddo
 end subroutine force_sub
 
 real*8 function pot(rr)
   implicit none
   real*8 rr(dim, na)
   
   !
   ! Here you will calculate the potential.  We have various choices. 
   ! I have implemented just a harmonic oscilator example;  you will implement
   ! The H2O potential or another potential
   !
   !

   pot = pot_harm(rr, dim, na)
!   pot = pot_h2o(rr, dim, na)    ! you'll need to write pot_h2o which
                                  ! call dpes_h2o.  NOTE that rr here is a 
                                  ! 3*3 vector, while the input to dpes_h2o,
                                  ! even though it has the same name, is very
                                  ! different -- it is a vector of 3 distances.

 end function pot
 
 real*8 function  pot_harm(rr, dim, na)
   implicit none
   
   integer    dim, na
   real*8  rr(dim, na), kappa, r_eq, r_abs
   
   kappa = 9.0 
   r_eq = 2.0
   r_abs = sqrt(sum((rr(:,1)-rr(:,2))**2))   ! dist. between two atoms

   pot_harm = kappa/2.d0* (r_abs-r_eq)**2   ! harmonic in dist.
 end function pot_harm


 
end program normal_modes
  

