!
! The program we had so far was not modular.  It mixed the propagation
! details such as : setting the parameters; propagating; 
! and the specific approach used for the propagation.
!
!
!  In order to make the program modular, we'll use a "storage space", or, as
!  fortran calls it, a module 
!
!  This version introduces the module; the next version shows how to 
!  properly use it to divide the program into different parts.
!
!
module constant_reactions
  implicit none
  save              ! keep this statement -- ensures vars. arent erased.

  integer, parameter :: n=4
  real*8, parameter :: kab = 1, kba = 3, kbc = 4.2, kcb = 7.3, kcd = 0.4
end module

program chemical_reactions_simple
  use constant_reactions
  implicit none

  real*8 dens(n)       
  real*8 ddens_dt(n)   
                       
  real*8:: a, b, c, d  
  real*8:: da_dt, db_dt, dc_dt, dd_dt

  real*8:: time, dt, tmax
  integer  nt, it
  
  dt = 0.0001;
  tmax = 60;     
  nt = tmax/dt;  
  
  dens = 0 
  dens(1) = 5

  time = 0;
  do it=1,nt

     a = dens(1)
     b = dens(2)
     c = dens(3)
     d = dens(4)

     time = time + dt;
     
     da_dt = -kab*a+kba*b;
     db_dt = +kab*a-kba*b -kbc*b +    kcb*c**2;
     dc_dt =               2*kbc*b -2*kcb*c**2 -kcd*c;
     dd_dt =                                   +kcd*c;
     
     ddens_dt(1) = da_dt  ! I could have avoided the use of da_dt but
     ddens_dt(2) = db_dt  ! This way the program is more transparent
     ddens_dt(3) = dc_dt
     ddens_dt(4) = dd_dt

     !
     ! Now comes the simple euler propagation--trivial in vector form
     !

     dens = dens + dt* ddens_dt
     
  end do

  write(6,*) &
  ' dt          time         a           b              c              d '
  write(6,'(1x,6e13.5)')dt,time,a,b,c,d

end
  








