!
! The program we had so far was not modular.  It mixed the propagation
! details such as : setting the parameters; propagating; 
! and the specific approach used for the propagation.
!
!
! OK, now we know what a module is; now we'll push all the parts
! which have to do with general propagation into a specific subroutine
!
!
module constant_reactions
  implicit none
  save              ! keep this statement -- ensures vars. arent erased.

  integer, parameter :: n=4
  real*8, parameter :: kab = 1, kba = 3, kbc = 4.2, kcb = 7.3, kcd = 0.4
end module

program chemical_reactions_simple
  use constant_reactions
  implicit none

  real*8 dens(n)       
  real*8 ddens_dt(n)   
                       
  real*8:: time, dt, tmax
  integer  nt, it
  
  external ddens_4spec_dt    ! This will be explained below
  dt = 0.0001;
  tmax = 60;     
  nt = tmax/dt;  
  
  dens = 0 
  dens(1) = 5

  time = 0;
  do it=1,nt

     !
     ! Note the structure of prop_euler.  We give it dens and its length
     !  (the vector of denities-- both input and output)
     !  We also give it dt, and, most importantly,
     !  we give it a name of a subroutine which will calculate
     !  ddens_dt from dens.  In our case, we call it ddens_4spec_dt.
     ! In order that the program knows that it is the name of a subroutine
     ! we need to declare it as "external" above.
     ! We also give it ddens_dt -- it needs to use that space.
     
     call prop_euler(dens, n, time, dt, ddens_4spec_dt, ddens_dt)
     
  end do

  write(6,*) &
  ' dt          time         a           b              c              d '
  write(6,'(1x,6e13.5)')dt,time,dens

end

! Note the following -- I'll explain in class, but briefly,
! we separate the propagation into two parts; one has to do with the
! propagation method (prop_euler); that routine calls a specifci routine
! (which prop_euler knows by the name of ddens_dt_sub; the actual name
! is passed to it, and is here ddens_4spec_dt)
!

subroutine prop_euler(dens, n, time, dt, ddens_dt_sub, ddens_dt)
  implicit none

  integer n
  real*8  dens(n), ddens_dt(n), time
  real*8  dt
  external ddens_dt_sub

  call ddens_dt_sub( time, dens, ddens_dt)   
  dens = dens + dt* ddens_dt
  
end

subroutine ddens_4spec_dt( time, dens, ddens_dt)
  use constant_reactions
  implicit none
  real*8 dens(n), ddens_dt(n), time
  real*8:: a, b, c, d  
  real*8:: da_dt, db_dt, dc_dt, dd_dt

  !
  ! Check
  !
  if(n/=4) then
     write(6,*)' n is not equal to 4 ',n
     stop
  endif

  a = dens(1)
  b = dens(2)
  c = dens(3)
  d = dens(4)
  
  da_dt = -kab*a+kba*b;
  db_dt = +kab*a-kba*b -kbc*b +    kcb*c**2;
  dc_dt =               2*kbc*b -2*kcb*c**2 -kcd*c;
  dd_dt =                                   +kcd*c;
  
  ddens_dt(1) = da_dt 
  ddens_dt(2) = db_dt 
  ddens_dt(3) = dc_dt
  ddens_dt(4) = dd_dt

end  
  
  







