! Now we are ready, more or less; the only thing needed is to 
! clean-up  Specifically, we'll build small modular subroutine
! that will read parameters.
!
! The reading of the parameters would be from an input file.
!
! We'll use several small routines, and, to avoid declaring unnecessarily
! variable names, etc., we'll use 'contain' -- as shown below.
!
! (ALSO: The next version would have a different propagation routine)
!
! The program is much longer, but much more easy to debug, change, etc.
!
!  
!
!

module constant_reactions
  implicit none
  save              
  integer, parameter :: n=4
  real*8             :: kab, kba, kbc, kcb, kcd 
end module

program chemical_reactions_simple
  use constant_reactions
  implicit none

  real*8 dens(n)       
  real*8 ddens_dt(n)   
                       
  real*8:: time, dt, tmax
  integer  nt, it
  
  external ddens_4spec_dt    ! This will be explained below
  
  call open_input   ! see below
  call read_times
  call read_initial_density
  call read_constants

  time=0.d0
  do it=1,nt
     time=time+dt
     call prop_euler(dens, n, time, dt, ddens_4spec_dt, ddens_dt)
  end do

  write(6,*) &
  ' dt          time         a           b              c              d '
  write(6,'(1x,6e13.5)')dt,time,dens

contains
  subroutine open_input
    implicit none   ! all the variables in the main routine
                    ! are recognized here
    open(unit=4,file='chem_rxtns.inp',status='old')  !self-evident
  end subroutine open_input

  subroutine read_times
    implicit none
    write(6,*)' give me  dt, tmax '
    read( 4,*)dt,tmax
    nt = tmax/dt
    write(6,*)' dt, tmax, nt = ',dt,tmax,nt   ! always write what you read
  end subroutine read_times

  subroutine read_initial_density
    implicit none

    write(6,*)' give me initial dens of length ',n
    read( 4,*)  dens
    write(6,*)' initial density = ',dens
  end subroutine read_initial_density

  subroutine read_constants
    implicit none

    write(6,*)' give me kab, kba, kbc, kcb, kcd '
    read( 4,*)          kab, kba, kbc, kcb, kcd 
    write(6,*)' = ',    kab, kba, kbc, kcb, kcd 
  end subroutine read_constants
end


subroutine prop_euler(dens, n, time, dt, ddens_dt_sub, ddens_dt)
  implicit none

  integer n
  real*8  dens(n), ddens_dt(n)
  real*8  time, dt
  external ddens_dt_sub

  call ddens_dt_sub( time, dens, ddens_dt)   
  dens = dens + dt* ddens_dt
  
end

subroutine ddens_4spec_dt( time, dens, ddens_dt)
  use constant_reactions
  implicit none
  real*8 dens(n), ddens_dt(n), time 
  real*8:: a, b, c, d  
  real*8:: da_dt, db_dt, dc_dt, dd_dt

  !
  ! Check
  !
  if(n/=4) then
     write(6,*)' n is not equal to 4 ',n
     stop
  endif

  a = dens(1)
  b = dens(2)
  c = dens(3)
  d = dens(4)
  
  da_dt = -kab*a+kba*b;
  db_dt = +kab*a-kba*b -kbc*b +    kcb*c**2;
  dc_dt =               2*kbc*b -2*kcb*c**2 -kcd*c;
  dd_dt =                                   +kcd*c;
  
  ddens_dt(1) = da_dt  ! I could have avoided the use of da_dt but
  ddens_dt(2) = db_dt  ! This way the program is more transparent
  ddens_dt(3) = dc_dt
  ddens_dt(4) = dd_dt

end  
  
  







