!  
! Here we'll worry about output of the densities as a function of time.
! A generic subroutine, plot_dens, written below.
!
! After running this program you'll get a series of files 
!   (fort.201 containing  t,[A](t),
!    fort.202 containing  t,[B}(t), etc.)
!
!  You may wonder why we dont use fort.101, fort.102, etc.  The reason
!  is thatm unless you declare explicitly, file nunmbers 5,6,101,102
!  are interpreted by fortran to mean the terminal.
!
!  After you run this subroutine, plot the results by e.g., writing
!
!      xmgr fort.201    (xmgrace fort.201  on linux machines)
!  or
!      xmgr fort.201 fort.202 fort.203 fort.204  
!  etc.
!
module constant_reactions
  implicit none
  save              
  integer, parameter :: n=4
  real*8             :: kab, kba, kbc, kcb, kcd 
end module

program chemical_reactions_simple
  use constant_reactions
  implicit none

  real*8 dens(n)       
  real*8 ddens_dt(n)   
                       
  real*8:: time, dt, tmax
  integer  nt, it, istep_plot
  
  external ddens_4spec_dt    
  
  call open_input   
  call read_times
  call read_initial_density
  call read_constants

  time=0.d0
  do it=1,nt
     time=time+dt
     call prop_euler(dens, n, time, dt, ddens_4spec_dt, ddens_dt)
     call plot_dens
  end do

  write(6,*) &
  ' dt          time         a           b              c              d '
  write(6,'(1x,6e13.5)')dt,time,dens

contains
  subroutine open_input
    implicit none   
                    
    open(unit=4,file='chem_rxtns.inp',status='old')  
  end subroutine open_input

  subroutine read_times     ! now we also read istep_plot
    implicit none
    write(6,*)' give me  dt, tmax, istep_plot '
    read( 4,*)  dt,tmax, istep_plot
    nt = tmax/dt
    write(6,*)' dt, tmax, nt, istep_plot = ',dt,tmax,nt, istep_plot   
  end subroutine read_times

  subroutine read_initial_density
    implicit none

    write(6,*)' give me initial dens of length ',n
    read( 4,*)  dens
    write(6,*)' initial density = ',dens
  end subroutine read_initial_density

  subroutine read_constants
    implicit none

    write(6,*)' give me kab, kba, kbc, kcb, kcd '
    read( 4,*)          kab, kba, kbc, kcb, kcd 
    write(6,*)' = ',    kab, kba, kbc, kcb, kcd 
  end subroutine read_constants

  subroutine plot_dens
    implicit none
    integer i
!
! This subroutine plots the densities once every step, or every tenth step,
! etc.  The decision on how often to plot is read.
! Every densitiy is printed in a file

    if(mod(it,istep_plot)==0)then  ! e.g., say istep_plot=30, then 
                                ! plotting would occur only when 
                                ! it=30,60,90, etc.
       !
       ! We'll plot dens(1) in file 201 (fort.201 unless we declare a diff.
       !                                 name)
       !
       ! dens(2) in file 202,
       ! dens(3) in file 203, etc.

       do i=1,N
          write(200+i,*)time, dens(i)
       enddo
    endif
  end subroutine plot_dens
end program


subroutine prop_euler(dens, n, time, dt, ddens_dt_sub, ddens_dt)
  implicit none

  integer n
  real*8  dens(n), ddens_dt(n), time 
  real*8  dt
  external ddens_dt_sub

  call ddens_dt_sub( time, dens, ddens_dt)   
  dens = dens + dt* ddens_dt
  
end

subroutine ddens_4spec_dt( time, dens, ddens_dt)
  use constant_reactions
  implicit none
  real*8 dens(n), ddens_dt(n), time
  real*8:: a, b, c, d  
  real*8:: da_dt, db_dt, dc_dt, dd_dt

  !
  ! Check
  !
  if(n/=4) then
     write(6,*)' n is not equal to 4 ',n
     stop
  endif

  a = dens(1)
  b = dens(2)
  c = dens(3)
  d = dens(4)
  
  da_dt = -kab*a+kba*b;
  db_dt = +kab*a-kba*b -kbc*b +    kcb*c**2;
  dc_dt =               2*kbc*b -2*kcb*c**2 -kcd*c;
  dd_dt =                                   +kcd*c;
  
  ddens_dt(1) = da_dt  ! I could have avoided the use of da_dt but
  ddens_dt(2) = db_dt  ! This way the program is more transparent
  ddens_dt(3) = dc_dt
  ddens_dt(4) = dd_dt

end  
  
  







