!
! Now we get to the final stage of the programming part.  We will read
! a "flag", i.e., an integer, that will tell us if we want to propagate
! with Euler's method or with a more sophisticated approach, Runge-Kutta's
! method.

module constant_reactions
  implicit none
  save              
  integer, parameter :: n=4
  real*8             :: kab, kba, kbc, kcb, kcd 
end module

program chemical_reactions_simple
  use constant_reactions
  implicit none

  real*8 dens(n)       
  real*8 ddens_dt(n)   
                       
  real*8:: time, dt, tmax
  integer  nt, it, istep_plot, flag_prop
  
  external ddens_4spec_dt    
  
  call open_input   
  call read_times
  call read_initial_density
  call read_constants
!
! Add this line to read
!
  write(6,*)' flag_prop= ? ';read(4,*)flag_prop; write(6,*)' =',flag_prop

  time=0.d0
  do it=1,nt
     time=time+dt
     call prop_generic
     call plot_dens
  end do

  write(6,*) &
  ' dt          time         a           b              c              d '
  write(6,'(1x,6e13.5)')dt,time,dens

contains
  subroutine open_input
    implicit none   
                    
    open(unit=4,file='chem_rxtns.inp',status='old')  
  end subroutine open_input

  subroutine read_times     
    implicit none
    write(6,*)' give me  dt, tmax, istep_plot '
    read( 4,*)  dt,tmax, istep_plot
    nt = tmax/dt
    write(6,*)' dt, tmax, nt, istep_plot = ',dt,tmax,nt, istep_plot   
  end subroutine read_times

  subroutine read_initial_density
    implicit none

    write(6,*)' give me initial dens of length ',n
    read( 4,*)  dens
    write(6,*)' initial density = ',dens
  end subroutine read_initial_density

  subroutine read_constants
    implicit none

    write(6,*)' give me kab, kba, kbc, kcb, kcd '
    read( 4,*)          kab, kba, kbc, kcb, kcd 
    write(6,*)' = ',    kab, kba, kbc, kcb, kcd 
  end subroutine read_constants

  subroutine prop_generic     ! a generic prop. routine that selects
                              ! which method should be used for prop.
    implicit none
    select case(flag_prop)    ! see class or ref. book for select --
    case(1)                ! a simple way to have multiple if's
       call prop_euler(dens, n, time,dt, ddens_4spec_dt, ddens_dt)
    case(2)
       call real_rk4_new4(dens,n,time,dt, ddens_4spec_dt) 
    case default
       write(6,*)' problem--flag_prop ',flag_prop; stop
    end select
  end subroutine


  subroutine plot_dens
    implicit none
    integer i

    if(mod(it,istep_plot)==0)then 
                                
       do i=1,N
          write(200+i,*)time, dens(i)
       enddo
    endif
  end subroutine plot_dens
end program


subroutine prop_euler(dens, n, time, dt, ddens_dt_sub, ddens_dt)
  implicit none

  integer n
  real*8  dens(n), ddens_dt(n), time
  real*8  dt
  external ddens_dt_sub

  call ddens_dt_sub( time, dens, ddens_dt)   
  dens = dens + dt* ddens_dt
  
end

subroutine ddens_4spec_dt( time, dens, ddens_dt)
  use constant_reactions
  implicit none
  real*8:: dens(n), ddens_dt(n), time
  real*8:: a, b, c, d  
  real*8:: da_dt, db_dt, dc_dt, dd_dt

  if(n/=4) then
     write(6,*)' n is not equal to 4 ',n
     stop
  endif

  a = dens(1)
  b = dens(2)
  c = dens(3)
  d = dens(4)
  
  da_dt = -kab*a+kba*b;
  db_dt = +kab*a-kba*b -kbc*b +    kcb*c**2;
  dc_dt =               2*kbc*b -2*kcb*c**2 -kcd*c;
  dd_dt =                                   +kcd*c;
  
  ddens_dt(1) = da_dt  
  ddens_dt(2) = db_dt  
  ddens_dt(3) = dc_dt
  ddens_dt(4) = dd_dt

end  
  
  







