!
! This file contains several subroutines. The fundamental one is the complex
! bessel. Also included is a real-arguement J_n and I_n, and a driver.
!

subroutine bessel_real_array_sub(bes, n, x)
  implicit none
  integer n, j, status
  real*8 x, bes(0:n)

  complex*16  c_x
  complex*16, dimension(:), allocatable :: c_bes

  allocate(c_bes(0:n), stat=status); call check(status, 0, ' real_bes_st ')
  c_x = x
  call c_bessel_sub(c_bes, n, c_x)  
  bes = c_bes;  deallocate(c_bes)

end subroutine bessel_real_array_sub

subroutine bessel_I_array_sub(bes_I, n, x)
  implicit none
  integer n, status, j
  real*8 x, bes_I(0:n)

  complex*16  c_x
  complex*16, parameter :: c_i = (0.d0, 1.d0)
  complex*16, dimension(:), allocatable :: c_bes

  allocate(c_bes(0:n), stat=status); call check(status, 0, ' real_bes_st ')

  c_x = x 
  call c_bessel_sub(c_bes, n, c_x)  
  do j=0, n
     bes_I(j) = 1.d0/(c_i**j) *c_bes(j)
  enddo
  deallocate(c_bes)

end subroutine bessel_I_array_sub
  

subroutine c_bessel_sub(c_bes, n, c_x)  ! c_bes(0:n) = cmplc_x c_bess of c
  implicit none
  integer n, n_big, status, j, l, i 
  
  complex*16 c_x
  complex*16 c_bes(0:n)
  complex*16, dimension(:), allocatable :: temp
  complex*16 bsum
  real*8     term

  n_big = max(2*n, n+100, int(2*abs(c_x)))

  if(abs(c_x) < 1.d-40) then
     c_bes = 0.d0
     c_bes(0) = 1.d0
     return
  end if

  if(n<0.or.n_big>200101) then
     write(6,*)' n, c_x ', n, c_x
     stop
  endif


  allocate(temp(0:n_big), stat = status); call check(status,0, ' c_bes_stat ')
  temp = 0.d0

  temp(n_big) =   0.d0
  temp(n_big-1) = 1.d0
  
  do j=n_big-2, 0, -1
     temp(j) = -temp(j+2) + 2.d0*(j+1.d0)/c_x*temp(j+1)
     if(abs(temp(j))>1000.d0) then 
        term = maxval(abs(temp))
        temp = temp/term
     endif
  enddo

  ! now normalize, using 
  bsum = temp(0)
  do j=1,n_big/2-2
     bsum = bsum + 2.d0* temp(2*j)
  enddo
  
  temp = temp/bsum
  if(sum(abs(temp(n_big-1:n_big)))> 1.d-13) then
     write(6,*)' problem; temp_last_ones '
     write(6,*) temp(n_big-1:n_big)
     stop
  endif

  c_bes = temp(0:n)
  deallocate(temp)
  
  write(92,*) ' c_bes '
  do j=0, n
     write(92,*)j,c_bes(j)
  enddo

end subroutine c_bessel_sub

subroutine bessel_driver
  implicit none
  integer    n, j
  real*8     x, y
  complex*16 c_x
  complex*16 c_bes(0:300)
  complex*16, parameter :: ci=(0.d0,1.d0)
  real*8     bes_real(0:300)
  real*8     bes_I(   0:300)
  write(6,*)' n, c_x '
  read( 5,*)  n,  x, y
  c_x = x + ci*y
  call        c_bessel_sub(c_bes,      n,    c_x)
  
  do j=0, n
     if(j<10.or.mod(j, n/6) ==0) then
        write(6,*)' j, c_bes ',j,c_bes(j)
     endif
  enddo

  call bessel_real_array_sub(bes_real, n,      x)
  do j=0, n
     if(j<10.or.mod(j, n/6) ==0) then
        write(6,*)' j, bes_real ',j,bes_real(j)
     endif
  enddo
   
  call bessel_I_array_sub(   bes_I,    n,      x)
  do j=0, n
     if(j<10.or.mod(j, n/6) ==0) then
        write(6,*)' j, bes_I ',j,bes_I(j)
     endif
  enddo
end subroutine bessel_driver






