!
! This file contains 
!  feit_fleck_new -- does propagation in one time step of
!   a kinteic hamiltonian in 3d (to use 1d, set ny=nz=1; for 2d nz=1)
!
! In addition, look in another file (library_driver_prop_3d_feit_fleck_new.f90)
! for driver_feit_fleck_new -- a complete program 
! 


subroutine feit_fleck_new_NS_option(cpsi, exp_vv_mi_dto2, &
     nx, massx, dx, &
     ny, massy, dy, &
     nz, massz, dz , &
     NS, dt, method)
  
  implicit none

  integer nx, ny, nz, ix, iy, iz, NS, is
  real*8  massx, massy, massz, dx, dy, dz, dt
  character*8 method
  
  complex*16 cpsi(nx, ny, nz, NS)
  complex*16, parameter :: ci = (0.d0, 1.d0)
  
  complex*16  exp_vv_mi_dto2(nx, ny, nz, NS, NS)
  
  call prop_v
  if(method=='realtime') then
     call prop_k_3d()
  else if(method=='imagtime') then
     call prop_k_3d_imag()
  else
     write(6,*)' method ',method
     stop
  endif
  call prop_v

contains
  SUBROUTINE prop_v
    implicit none
    
    do ix=1, nx
       do IY=1, NY
          do iz=1, NZ
             cpsi(ix, iy, iz, :) = &
             matmul(exp_vv_mi_dto2(ix, iy, iz, :, :), cpsi(ix, iy, iz, :))
             
          enddo
       enddo
    enddo
    
  end SUBROUTINE prop_v
  
  subroutine prop_k_3d()
    implicit none
    complex*16 c1d(2000)
    
    call check_le(max(nx, ny, nz),  2000, 'mnxyz2   ')
    
    DO IS=1, NS
       
       do iz=1,nz
          do iy=1,ny
             if(nx>1) then
                c1d(1:nx) = cpsi(1:nx, iy, iz, is)
                
                call fft_prop_1d_good(c1d, nx, massx, dx, dt) ! out could=in.
                cpsi(1:nx, iy, iz, is) = c1d(1:nx)
             endif
          enddo
       enddo
       
       do iy=1, ny
          do ix=1, nx
             if(Nz>1) then
                c1d(1:nz) = cpsi(ix,iy,1:nz, is)
                
                call fft_prop_1d_good(c1d, nz, massz, dz, dt) 
                cpsi(ix, iy, 1:nz, is) = c1d(1:nz)
             endif
          enddo
       enddo
       
       do iz=1, nz
          do ix=1,nx
             if(ny>1) then
                c1d(1:ny) = cpsi(ix, 1:ny, iz, is) 
                
                call fft_prop_1d_good(c1d, ny, massy, dy, dt) 
                cpsi(ix, 1:ny, iz, is) = c1d(1:ny)
             endif
          enddo
       enddo
       
    enddo
  end subroutine prop_k_3d

  subroutine prop_k_3d_imag()
    implicit none
    complex*16 c1d(2000)
    
    call check_le(max(nx, ny, nz),  2000, 'mnxyz2   ')
    
    DO IS=1, NS
       
       do iz=1,nz
          do iy=1,ny
             if(nx>1) then
                c1d(1:nx) = cpsi(1:nx, iy, iz, is)
                
                call fft_prop_imag_1d_good(c1d, nx, massx, dx, dt) ! out could=in.
                cpsi(1:nx, iy, iz, is) = c1d(1:nx)
             endif
          enddo
       enddo
       
       do iy=1, ny
          do ix=1, nx
             if(Nz>1) then
                c1d(1:nz) = cpsi(ix,iy,1:nz, is)
                
                call fft_prop_imag_1d_good(c1d, nz, massz, dz, dt) 
                cpsi(ix, iy, 1:nz, is) = c1d(1:nz)
             endif
          enddo
       enddo
       
       do iz=1, nz
          do ix=1,nx
             if(ny>1) then
                c1d(1:ny) = cpsi(ix, 1:ny, iz, is) 
                
                call fft_prop_imag_1d_good(c1d, ny, massy, dy, dt) 
                cpsi(ix, 1:ny, iz, is) = c1d(1:ny)
             endif
          enddo
       enddo
       
    enddo
  end subroutine prop_k_3d_imag

end 
  












