
!
! driver and example are at the bottom
!

module bessel_eres_cheby_module
  implicit none
  save
  complex*16, dimension(:), allocatable  :: c_bes_coef
  complex*16, allocatable, dimension(:,:) :: cecof

end module bessel_eres_cheby_module

!
!  c_ket_fin can be the same as c_ket
!

subroutine &
prop_cheby_ket_e_resolv(c_ket, c_ket_fin, size_ket, &
     tmin, tmax, propagation_type, &
     hmin, hmax, hsub,  &
     ne, c_ket_resolv, energy)
  use bessel_eres_cheby_module
  implicit none
  
  external   hsub
  integer    ne
  integer    size_ket
  integer,   parameter :: n_big = 10000
  real*8     dt, hmin, hmax  
  real*8     tmin, tmax
  real*8     energy(ne)
  complex*16 c_ket(    size_ket)
  complex*16 c_ket_fin(size_ket)
  complex*16 c_ket_resolv(size_ket, ne)

  complex*16, parameter :: ci = (0.d0, 1.d0)

  character*9 propagation_type, propagation_type_old

  integer     kmax, ktop, j, k, ie
  real*8      havg, dh
  real*8      havg_old, dh_old, dt_old
  integer     ifirst 
  data        ifirst / 1 /
  save        ifirst, havg_old, dh_old, dt_old, kmax, ktop
  save        propagation_type_old

  complex*16, allocatable, dimension(:)   :: tkA, tkB, tkC

  dt = tmax - tmin


  call check(abs(ifirst), 1, ' abs_ifirst ')
  havg = (hmax+hmin)/2.d0
  dh =   (hmax-hmin)/2.d0
  dh =   dh*2.d0
  kmax = dh*dt*2+100
  kmax = kmax - mod(kmax, 4)-1   ! needs to be divided by 4 with rem. 3
  call check_le(kmax, n_big, ' kmax, big ')

  ! calculate c_bes_coef only if necessary

  update : if( ifirst == 1  .or.  &
       (ifirst == -1 .and.&
       (abs(dt -dt_old  ) > 1.d-8   .or. &
       abs(dh  -dh_old  ) > 1.d-8   .or. &
       abs(havg-havg_old) > 1.d-8   .or. &
       propagation_type /= propagation_type_old))) then
     
     if(ifirst /= 1) then
       
        deallocate(cecof, c_bes_coef, stat=j); call check(j,0,' c_deall ')
     endif

     allocate(c_bes_coef(0:n_big), stat=j); if(j/=0) stop
     call prep_cheby_coef(c_bes_coef,kmax,ktop,dh,havg,dt,propagation_type)
     
     if(ne>0) then
        allocate(cecof(ne, 0:ktop),  stat=j);  if(j/=0) stop
        call RESCOPRE_NEW(ne,energy,ktop, cecof, dt, havg, dh)
     end if

  end if update

  dt_old = dt
  dh_old = dh
  havg_old = havg
  propagation_type_old = propagation_type
  ifirst = -1

  ! now can get ket_fin(dt) = sum_k c_bes_coef(k) T_k(H) ket

  allocate(tkA(size_ket), stat = j); if(j/=0) stop
  allocate(tkB(size_ket), stat = j); if(j/=0) stop
  allocate(tkC(size_ket), stat = j); if(j/=0) stop
  
  tkA = c_ket
  call hscl_psi(tkA, tkB, size_ket, havg, dh, hsub)
  c_ket_fin = c_bes_coef(0)*tkA + c_bes_coef(1)*tkB

  if(ne>0) then
     call eresadd_new(energy,ne,size_ket, ktop, tkA, 0,c_ket_resolv,cecof,tmin)
     call eresadd_new(energy,ne,size_ket, ktop, tkB, 1,c_ket_resolv,cecof,tmin)
  endif

  do k=2, ktop
     
     call hscl_psi(tkB, tkC, size_ket, havg, dh, hsub)
     
     tkC = 2.d0*tkC-tkA

     c_ket_fin = c_ket_fin + tkC*c_bes_coef(k)
     if(ne>0) then
        call eresadd_new(energy,ne,size_ket,ktop,tkC,k,c_ket_resolv,cecof,tmin)
     endif

     tkA = tkB
     tkB = tkC

  enddo


  
  deallocate(tkA, tkB, tkC)

end subroutine

subroutine prep_cheby_coef(       c_bes_coef, kmax, ktop, dh, havg, dt, &
                                  propagation_type)
  implicit   none
  real*8     dt, havg, dh  ! can have complex
  integer    kmax, ktop,k 
  complex*16 c_bes_coef(0:kmax), c_factor
  
  real*8,     parameter :: tol = 1.d-16
  complex*16  c_x
  complex*16, parameter ::  c_i = (0.d0, 1.d0)

  character*9 propagation_type

  if (    propagation_type == 'real_time') then
     c_x = dh*dt
     c_factor = exp(-c_i* havg* dt)
  else if (propagation_type == 'imag_time') then
     c_x = -c_i*dh*dt
     c_factor = exp(-havg*dt)
  else
     write(6,*)' problem type ', propagation_type;     stop
  endif

  call c_bessel_sub(c_bes_coef, kmax, c_x) 

  do k=0+1, kmax
     c_bes_coef(k) = c_bes_coef(k)  * 2.d0  !   *(-c_i)**k
  enddo

  do k=0,kmax-3,4
      c_bes_coef(k+1) =c_bes_coef(k+1)*(-c_i)
   enddo

   do k=0,kmax-3,4
      c_bes_coef(k+2) =c_bes_coef(k+2)*(-1)
   enddo

  do k=0,kmax-3,4
      c_bes_coef(k+3) = c_bes_coef(k+3)/(-c_i)!compiler crazy if *(c_i)
  enddo

  c_bes_coef = c_factor * c_bes_coef ! leave outside prevs loop for j=0 term

  do ktop=kmax,0,-1
     if(abs(c_bes_coef(ktop))> tol) exit
  enddo
  call check_le(1,ktop,' cheby_1_ktop ')

  write(6,*) ' kmax, ktop ', kmax, ktop  

  do k=0,ktop
    write(6,*)k,c_bes_coef(k)
  enddo

end subroutine prep_cheby_coef
  
subroutine hscl_psi(tkA, tkB, size_ket, havg, dh, hsub)
  implicit none
  external hsub
  integer  size_ket
  real*8 dh, havg
  complex*16 tkA(size_ket)
  complex*16 tkB(size_ket)
  
  call hsub( tkA, tkB, size_ket)
  
  tkB = tkB/dh - havg /dh*tkA

end subroutine hscl_psi

! ------------------------------------------------------------------
!   rest of file is a driver and a simple sub. for checking
! ------------------------------------------------------------------







