program main
  !
  ! implicit none    :means that every variable has to be declared;
  !             This way you avoid problems with spelling mistakes, e.g.,
  !             if you call a variable somewhere tfinal and sometimes else 
  !             tifnal, the compiler will detect it. 
  
  implicit none           ! The program always starts with this statement
  
  ! We now need to definw the variables we'll use.
  !
  !
  ! real*8  : means that we deal with a high accuracy real-number, 
  !           e.g., -35.234506798023  or 2.34567812845*10**(-10)
  !
  real*8 x          ! The value at which we check
  real*8 dx         ! The spacing used in the derivative formula
  real*8 fx         ! This variable will store f(x)
  real*8 fxpr       ! This varaible will store f(x+dx)
  real*8 fxdrv_a    ! This variable will store the analytic drv. of f at x
  real*8 fxdrv_a_pr ! This variable will store the analytic drv. of f at x+dx
  real*8 fx_num_E1  ! This variable will store the 1st order Euler 
                    !                  numeric  drv. of f at x
  
  x  = 0.3   ! An arbitray value where we check.
  dx = 2d-7  ! An abirtrary dx. Means: 2*10**(-7) (x**y mean x to the power y)
  
  
  !
  ! gauss_sub is an abritray subroutine I wrote below which
  ! calculates a function (here exp(-x**2/2) and its analytic derivative.
  ! I could have picked any other function for this demonstration.
  !
  call gauss_sub(x,    fx,   fxdrv_a)   
  call gauss_sub(x+dx, fxpr, fxdrv_a_pr)
  
  !
  !  Euler's formula for the derivative
  !
  
  fx_num_E1 = (fxpr-fx)/dx      
  
  !
  ! Now I will print the numerical result and its difference from the analytic
  ! result
  
  print *,' df/dx(Euler),  df/dx(Euler) - df/dx(Analytical) '
  print *,       fx_num_E1, fx_num_E1   - fxdrv_a
  
  !
  ! The main program and each subroutine needs to finish with: end
  !
end program main

!
!
!  Subroutine is a piece of program that get's some 
!  variables, uses the information in the variables and possibly modifys
!  them.
!
!  Note that purely input variables can be expressions (such as 
!  when we call this subroutine with an arumgent x+dx above); but in that
!  case we cannot modify these variables, or the program will go beserk.
!
!

subroutine gauss_sub(x, fx, fxdrv_a)
  implicit none

  real*8 x, fx, fxdrv_a

  fx      =      exp(-x**2/2)     
  fxdrv_a = -x*  exp(-x**2/2)     

end subroutine gauss_sub

!  Exercises 
!  (i) Run this program with several different values of h, starting
!  at h=1e-14, 1e-12, ...,1e-4,1e-2.  Do you see a plataue?  What
!  happens at very low values of h and very high values of h.  
!  (Running like this is painstaking; the next part would automate it) 
























