!
!   In part 1, we changed dx by hand, which was painstaking.
!   Here, we'll do it automatically, by looping over different values.
!   This will be a chance to learn about loops.
!
!   Note the structure of the loop.  It starts with a "do", then
!       the name of an index (j here), 
!       and then its min and max values, separated
!       by a comma   (the index rises by 1 each time; there are ways to
!               change the step size, but let's not worry about it)
!   The loop is indented (not obligaroty, but helps reading)
!   And ends with "enddo"
!
!   The one remaining problem here is that the output is not looking nice.
!   We could have worried about formatting, but I wont worry about it.
!
!   See also notes below.
!

program main
  implicit none         

  real*8 x          
  real*8 dx         
  real*8 fx         
  real*8 fxpr       
  real*8 fxdrv_a    
  real*8 fxdrv_a_pr 
  real*8 fx_num_E1  
           
  integer j          ! j is an index, so it is of "type" integer; it 
                     ! is strictly 0, or +1 or -1 or +2 or -2, etc.
  
  x  = 0.3   
  
  print *,' dx,  df/dx(Euler) - df/dx(Analytical) '   ! now we'll print dx
  do j=1,20
                         ! Note indentation.

     dx = 10.d0**(-dble(j))       ! So dx = 0.1, 0.01, 0.001, etc.
                                  ! To see the meaning of the dble see below.

     call gauss_sub(x,    fx,   fxdrv_a)   
     call gauss_sub(x+dx, fxpr, fxdrv_a_pr)
     
     fx_num_E1 = (fxpr-fx)/dx      
     
     print *,  dx,  fx_num_E1   - fxdrv_a
  enddo
end  


subroutine gauss_sub(x, fx, fxdrv_a)
  implicit none

  real*8 x, fx, fxdrv_a

  fx      =      exp(-x**2/2)     
  fxdrv_a = -x*  exp(-x**2/2)     

end subroutine gauss_sub

!
!  Notes: every programming language has its pitfalls; so does fortran90.
!  The first one is that you have to be careful when dealing with 
!  integer.  For example, if you write
!
!  x= 2/4, 
!
!  Then fortran will do INTEGER algebra for the 2/4, and think of it as 
!  zero, so the result would be zero.  
!
!  Similarly, if we were to 
!  Calculate dx above as 10**(-j), then fortran would do integer algbera for
!  it, and the result (for j=1,2, etc.) would be zero.
!
!  So the first step is to convert integers in expressions to real (acutally
!  real*8) variables for greater accuracy -- that's what dble does.
!
!  So we could have rewritten 10**(-j) as dble(10)**dble(-j).
!
!  But it is easier to write 10 directly as a real*8 varibale, i.e., as
!  10.d0    (meaning 10, with exponet 0 in base 10; the d means that
!            we're using double precision).
!  Other examples are 12.3d0 (which equals 1.23d1, etc.)









