!
! Part 3: 
!  Here we'll first learn about "formatted writing" (see below), so to make the
!  output look nice


program main
  implicit none         

  real*8 x          
  real*8 dx         
  real*8 fx         
  real*8 fxpr       
  real*8 fxdrv_a    
  real*8 fxdrv_a_pr 
  real*8 fx_num_E1  
           
  integer j          
                     
  
  x  = 0.3   
  
  print *,' dx,  df/dx(Euler) - df/dx(Analytical) '   
  do j=1,20
                         ! Note indentation.

     dx = 10.d0**(-dble(j))       
                                  

     call gauss_sub(x,    fx,   fxdrv_a)   
     call gauss_sub(x+dx, fxpr, fxdrv_a_pr)
     
     fx_num_E1 = (fxpr-fx)/dx      
     
     write(6,'(d15.5,d17.5)')  dx,  fx_num_E1   - fxdrv_a   ! see below
  enddo
end  

! Formatted writing.  It should only be done when the program is more
! or less working and you want to beutify the results.
!
! The statement we have  is:
!     write(6,'(1x, d15.5,d17.5) ') variable1, variable2
!
! The first thing is the "6".  It refers to the file index; in fortran
! 6 is reserved to output, and 5 to input.
!
! Next is the '(1x,d15.5,d17.5)' -- the format.  It always starts with
! '(  and ends with )'
!
! Inside, the 1x means that fortran should skip a space (do it always,
!  for historical safety reasons),
! and then that we want to print two real variables.  The first is printted
! here with a d15.5 format, i.e., save 15 spaces for the variable, and have
! 5 digits after the decimal dot.
!
! We can play with the numbers in the d15.5, use instead d10.3, etc. -- but 
! always make sure the first one (the total number of spaces) is large enough
! to contain the variables.

!
! Other formats you may use:  e  (same as d)
!                             i   (for integers, e.g.,  i8 or i6)
!                             a   (for characters -- later)

! 

subroutine gauss_sub(x, fx, fxdrv_a)
  implicit none

  real*8 x, fx, fxdrv_a

  fx      =      exp(-x**2/2)     
  fxdrv_a = -x*  exp(-x**2/2)     

end subroutine gauss_sub

!
!  Notes: every programming language has its pitfalls; so does fortran90.
!  The first one is that you have to be careful when dealing with 
!  integer.  For example, if you write
!
!  x= 2/4, 
!
!  Then fortran will do INTEGER algebra for the 2/4, and think of it as 
!  zero, so the result would be zero.  
!
!  Similarly, if we were to 
!  Calculate dx above as 10**(-j), then fortran would do integer algbera for
!  it, and the result (for j=1,2, etc.) would be zero.
!
!  So the first step is to convert integers in expressions to real (acutally
!  real*8) variables for greater accuracy -- that's what dble does.
!
!  So we could have rewritten 10**(-j) as dble(10)**dble(-j).
!
!  But it is easier to write 10 directly as a real*8 varibale, i.e., as
!  10.d0    (meaning 10, with exponet 0 in base 10; the d means that
!            we're using double precision).
!  Other examples are 12.3d0 (which equals 1.23d1, etc.)









